#ifndef CORE_TEST_HPP
#define CORE_TEST_HPP

#include "core/Logger.hpp"

namespace Core {
    void finalizeTests(void);
    bool addTestResult(const char* file, bool comparison);

    template<typename T>
    bool testEqual(
        const char* file, int line, const T& wanted, const T& actual) {
        file = getShortFileName(file);
        if(addTestResult(file, wanted == actual)) {
            return true;
        }
        char buffer[512];
        formatBuffer(
            buffer, sizeof(buffer),
            TERMINAL_RED "#:# - expected '#' got '#'" TERMINAL_RESET, file,
            line, wanted, actual);
        puts(buffer);
        return false;
    }

    bool testString(
        const char* file, int line, const char* wanted, const char* actual);

    template<typename A, typename B>
    bool testString(
        const char* file, int line, const A& wanted, const B& actual) {
        char wantedString[512];
        size_t lw = toString(wanted, wantedString, sizeof(wantedString));
        char actualString[512];
        size_t la = toString(actual, actualString, sizeof(actualString));
        testEqual(file, line, lw, la);
        return testString(
            file, line, static_cast<const char*>(wantedString),
            static_cast<const char*>(actualString));
    }

    bool testFloat(
        const char* file, int line, float wanted, float actual, float error);
    bool testNull(const char* file, int line, const void* p);
    bool testNotNull(const char* file, int line, const void* p);

#define TEST_FLOAT(wanted, actual, error)                      \
    Core::testFloat(__FILE__, __LINE__, wanted, actual, error)

#define TEST(wanted, actual)                                  \
    Core::testEqual<Core::RemoveReference<decltype(actual)>>( \
        __FILE__, __LINE__, wanted, actual)
#define TEST_STRING(wanted, actual)                      \
    Core::testString(__FILE__, __LINE__, wanted, actual)
#define TEST_FALSE(actual) TEST(false, actual)
#define TEST_TRUE(actual) TEST(true, actual)
#define TEST_NULL(actual) Core::testNull(__FILE__, __LINE__, actual)
#define TEST_NOT_NULL(actual) Core::testNotNull(__FILE__, __LINE__, actual)
}

#endif
