#include "core/Box.hpp"

#include <cstdio>

using Core::Box;

Box::Box(const Vector3& min_, const Vector3& max_) : min(min_), max(max_) {
}

Box::Box(const Vector3& size) : min(), max() {
    for(size_t i = 0; i < 3; i++) {
        if(size[i] < 0.0f) {
            min[i] = size[i];
        } else {
            max[i] = size[i];
        }
    }
}

Box Box::offset(const Vector3& offset) const {
    return Box(min + offset, max + offset);
}

bool Box::collidesWith(const Box& other) const {
    return max[0] > other.min[0] && min[0] < other.max[0] &&
           max[1] > other.min[1] && min[1] < other.max[1] &&
           max[2] > other.min[2] && min[2] < other.max[2];
}

Box Box::expand(const Vector3& offset) const {
    Vector3 add(
        offset[0] > 0.0f ? offset[0] : 0.0f,
        offset[1] > 0.0f ? offset[1] : 0.0f,
        offset[2] > 0.0f ? offset[2] : 0.0f);
    Vector3 sub(
        offset[0] < 0.0f ? offset[0] : 0.0f,
        offset[1] < 0.0f ? offset[1] : 0.0f,
        offset[2] < 0.0f ? offset[2] : 0.0f);
    return Box(min + sub, max + add);
}

Box Box::grow(const Vector3& growth) const {
    Vector3 half = growth * 0.5f;
    Vector3 nMin = min - half;
    Vector3 nMax = max + half;
    for(size_t i = 0; i < 3; i++) {
        if(nMin[i] > nMax[i]) {
            nMin[i] = (min[i] + max[i]) * 0.5f;
            nMax[i] = nMin[i];
        }
    }
    return Box(nMin, nMax);
}

const Core::Vector3& Box::getMin() const {
    return min;
}

const Core::Vector3& Box::getMax() const {
    return max;
}

size_t Box::toString(char* s, size_t n) const {
    int w = snprintf(
        s, n, "Box([%.2f, %.2f, %.2f], [%.2f, %.2f, %.2f])",
        static_cast<double>(min[0]), static_cast<double>(min[1]),
        static_cast<double>(min[2]), static_cast<double>(max[0]),
        static_cast<double>(max[1]), static_cast<double>(max[2]));
    return w >= 0 ? static_cast<size_t>(w) : 0;
}
