#include "core/View.hpp"

using Core::View;

void View::updateDirections(float lengthAngle, float widthAngle) {
    setAngles(front, lengthAngle, widthAngle);
    right = cross(front, Vector3(0.0f, 1.0f, 0.0f));
    right.normalize();
    up = cross(right, front);
    up.normalize();
    left = -right;
    back = -front;
    down = -up;
}

void View::updateDirections(const Quaternion& q) {
    up = q * Vector3(0.0f, 1.0f, 0.0f);
    back = q * Vector3(-1.0f, 0.0f, 0.0f);
    right = cross(up, back);
    right.normalize();
    left = -right;
    front = -back;
    down = -up;
}

const Core::Matrix& View::updateMatrix(const Vector3& pos) {
    view.set(0, Vector4(right[0], right[1], right[2], -right.dot(pos)));
    view.set(1, Vector4(up[0], up[1], up[2], -up.dot(pos)));
    view.set(2, Vector4(back[0], back[1], back[2], -back.dot(pos)));
    view.set(3, Vector4(0.0f, 0.0f, 0.0f, 1.0f));
    return view;
}

const Core::Vector3& Core::View::getUp() const {
    return up;
}

const Core::Vector3& Core::View::getDown() const {
    return down;
}

const Core::Vector3& Core::View::getLeft() const {
    return left;
}

const Core::Vector3& Core::View::getRight() const {
    return right;
}

const Core::Vector3& Core::View::getFront() const {
    return front;
}

const Core::Vector3& Core::View::getBack() const {
    return back;
}
